#include <iostream>
#include <types/model.h>

void Model::loadOBJ(const std::string& filename) {
    std::ifstream file(filename);
    //if (!file.is_open())
    std::vector<Vector3> positions;
    std::vector<Vector2> uvs;
    TextureInformation tInfo = {};
    std::string line;
    while (std::getline(file, line)) {
        std::istringstream stream(line);
        std::string prefix;
        stream >> prefix;

        if (prefix == "v") {
            float x, y, z;
            stream >> x >> y >> z;
            positions.emplace_back(x, y, z);
        } else if (prefix == "vt") {
            float u, v;
            stream >> u >> v;
            uvs.emplace_back(u, v);
        } else if (prefix == "f") {
            unsigned int vertexIndex[3], texCoordIndex[3];
            char slash;
            for (int i = 0; i < 3; ++i) {
                stream >> vertexIndex[i] >> slash >> texCoordIndex[i];
                vertexIndex[i]--;
                texCoordIndex[i]--;
            }

            for (int i = 0; i < 3; ++i) {
                Vertex vertex;
                Vector2 textureCoordinate;
                vertex = positions[vertexIndex[i]];
                textureCoordinate = uvs[texCoordIndex[i]];

                vertices.push_back(vertex);
                tInfo.textureCoordinates.push_back(textureCoordinate);
                indices.push_back(static_cast<unsigned int>(indices.size()));
            }
        }
    }
    textureInfo.push_back(tInfo);
    file.close();
};

// map face of model to positions and texture coords
void Model::mapFaces(const FaceIndices& facedata, const std::vector<Vector3>& positions, const std::vector<Vector2>& uvs) {
    std::vector<Vector2> textureCoordinates;
    for (int i = 0; i < 3; ++i) {
        Vector3 vertex;
        //Vector2 textureCoordinate;
        vertex = positions[facedata.vertexIndex[i]];
        //textureCoordinate = uvs[facedata.texCoordIndex[i]];
        vertices.push_back(vertex);
        indices.push_back(static_cast<unsigned int>(indices.size()));
        //textureCoordinates.push_back(textureCoordinate);
    }
    textureInfo.push_back({"", textureCoordinates});
}